package com.tanpu.community.manager;

import com.tanpu.common.api.CommonResp;
import com.tanpu.common.exception.BizException;
import com.tanpu.community.api.beans.req.theme.ForwardThemeReq;
import com.tanpu.community.api.beans.qo.AttachmentQo;
import com.tanpu.community.api.beans.qo.MainTextQo;
import com.tanpu.community.api.beans.qo.ThemeQo;
import com.tanpu.community.api.beans.req.theme.CreateThemeReq;
import com.tanpu.community.api.beans.req.theme.ThemeListByTopicReq;
import com.tanpu.community.api.beans.req.theme.ThemeListReq;
import com.tanpu.community.api.enums.BlockTypeEnum;
import com.tanpu.community.api.enums.CollectionTypeEnum;
import com.tanpu.community.api.enums.ThemeTypeEnum;
import com.tanpu.community.dao.entity.community.BlackListEntity;
import com.tanpu.community.dao.entity.community.HomePageEntity;
import com.tanpu.community.dao.entity.community.ThemeAttachmentEntity;
import com.tanpu.community.dao.entity.community.ThemeEntity;
import com.tanpu.community.feign.course.FeignClientForCourse;
import com.tanpu.community.api.beans.vo.feign.CourseDetailExt;
import com.tanpu.community.feign.product.FeignClientForProducts;
import com.tanpu.community.feign.tanpuroom.FeignClientForTanpuroom;
import com.tanpu.community.api.beans.vo.feign.ProductInfoVO;
import com.tanpu.community.api.beans.vo.feign.ShortVideoBaseInfoResp;
import com.tanpu.community.feign.zhibo.FeignClientForZhibo;
import com.tanpu.community.api.beans.vo.feign.ZhiboDetailVO;
import com.tanpu.community.service.*;
import com.tanpu.community.service.other.*;
import com.tanpu.community.util.ConvertUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.Duration;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

@Service
public class ThemeManager {
    @Resource
    private ThemeService themeService;

    @Autowired
    private CollectionService collectionService;

    @Autowired
    private CommentService commentService;

    @Autowired
    private FollowRelService followRelService;

    @Autowired
    private BlackListService blackListService;

    @Autowired
    private FinProResService finProResService;

    @Autowired
    private RedisService redisService;

    @Autowired
    private ZhiboService zhiboService;

    @Autowired
    private CurriculumResService curriculumResService;

    @Autowired
    private OrderFlowService orderFlowService;

    @Autowired
    private ThemeAttachmentService themeAttachmentService;

    @Autowired
    private HomePageService homePageService;

    @Resource
    private FeignClientForTanpuroom feignClientForTanpuroom;

    @Resource
    private FeignClientForZhibo feignClientForZhibo;

    @Resource
    private FeignClientForProducts feignClientForProducts;

    @Autowired
    private FeignClientForCourse feignClientForCourse;



    public void publishTheme(CreateThemeReq req, String userId) {
        ThemeEntity themeEntity = new ThemeEntity();
        BeanUtils.copyProperties(req, themeEntity);
        themeEntity.setAuthorId(userId);

        themeService.insertTheme(themeEntity);
        List<ThemeAttachmentEntity> themeAttachments = ConvertUtil.themeReqToAttachmentList(req, themeEntity.getThemeId());
        //TODO 敏感词过滤
        themeAttachmentService.insertList(themeAttachments);

    }

    // 返回推荐主题文章
    public List<ThemeQo> selectHotThemes(ThemeListReq req, String userId) {
        // TODO:推荐
        List<ThemeEntity> themeEntities = themeService.selectAll(req.getLastId(),req.getPageSize());
        List<ThemeQo> themeQos = convertEntityToQo(themeEntities, userId);
        return themeQos;
    }


    // 返回关注主题
    public List<ThemeQo> selectInterestThemes(ThemeListReq req,String userId) {
        List<String> fansList = followRelService.queryFansByFollowerId(userId);
        List<ThemeEntity> themeEntities = themeService.queryByFans(fansList,req.getLastId(),req.getPageSize());
        List<ThemeQo> themeQos = convertEntityToQo(themeEntities, userId);
        return themeQos;
    }

    public List<ThemeQo> queryByTopic(ThemeListByTopicReq req, String userId) {
        //TODO
        List<ThemeEntity> themeEntities = themeService.queryByTopic(req.getTopicId(),req.getLastId(),req.getPageSize());
        List<ThemeQo> themeQos = convertEntityToQo(themeEntities, userId);
        return themeQos;
    }

    //返回正文
    public MainTextQo getMainText(String themeId, String userId) {

        ThemeEntity themeEntity = themeService.queryById(themeId);
        if (themeEntity == null) {
            throw new BizException("找不到帖子id：" + themeId);
        }
        MainTextQo mainTextQo = new MainTextQo();
        BeanUtils.copyProperties(themeEntity, mainTextQo);
         buildMainTestExtraInfo(mainTextQo, userId);
        return mainTextQo;
    }

    // 点赞
    public void like(String themeId, String userId) {
        collectionService.addIfNotExist(themeId, userId, CollectionTypeEnum.LIKE_THEME);
    }

    public void unlike(String themeId, String userId) {
    }



    //转发
    public void forward(ForwardThemeReq req, String userId) {
        ThemeEntity targetTheme = themeService.queryById(req.getFormerThemeId());
        ThemeEntity newTheme = ThemeEntity.builder()
                .content(req.getContent())
                .topicId(req.getTopicId())
                .formerThemeId(req.getFormerThemeId())
                .authorId(userId)
                .themeType(ThemeTypeEnum.DISCUSSION.getCode())
                .build();

        themeService.insertTheme(newTheme);
    }


    //收藏
    public void favorite(String themeId, String userId) {
        collectionService.addIfNotExist(themeId, userId, CollectionTypeEnum.BOOK_THEME);
    }

    public void unFavorite(String themeId, String userId) {
        collectionService.delete(themeId, userId, CollectionTypeEnum.BOOK_THEME);
    }

    //投诉（主题）
    public void complaint(String themeId, String user) {
        //TODO
    }

    //    屏蔽(用户)
    public void blockUser(String blockUser, String userId) {
        BlackListEntity selectOne = blackListService.selectOne(blockUser, userId, BlockTypeEnum.USER.getCode());
        if (selectOne == null) {
            blackListService.addBlock(blockUser, userId, BlockTypeEnum.USER);
        }
    }

    // 解除屏蔽(用户)
    public void unblockUser(String blockUser, String userId) {
        //todo
    }

    // 屏蔽(主题)
    public void blockTheme(String themeId, String userId) {
        blackListService.addBlock(themeId, userId, BlockTypeEnum.THEME);
        BlackListEntity selectOne = blackListService.selectOne(themeService.queryById(themeId).getAuthorId(), userId, BlockTypeEnum.USER.getCode());
        if (selectOne == null) {
            blackListService.addBlock(themeService.queryById(themeId).getAuthorId(), userId, BlockTypeEnum.USER);
        }
    }

    // 解除屏蔽(主题)
    public void unblockTheme(String themeId, String userId) {
        blackListService.removeBlackList(themeId, userId, BlockTypeEnum.USER);
    }


    //主题Entity转QO
    private List<ThemeQo> convertEntityToQo(List<ThemeEntity> themeEntities, String userId) {
        List<ThemeQo> themeQos = ConvertUtil.themeEntitiesToDTOs(themeEntities);
        for (ThemeQo themeQO : themeQos) {
            buildThemeQoExtraInfo(userId, themeQO);
        }
        return themeQos;
    }


    //组装主题列表
    private void buildThemeQoExtraInfo(String userId, ThemeQo themeQO) {
        //附件列表
        String themeId = themeQO.getThemeId();
        List<ThemeAttachmentEntity> attachments = themeAttachmentService.selectByThemeId(themeId);
        if (!CollectionUtils.isEmpty(attachments)) {
            ThemeAttachmentEntity attachmentEntity = attachments.get(0);
            AttachmentQo attachmentQo = AttachmentQo.builder().type(attachmentEntity.getAttachType()).attachment(transferAttachment(attachmentEntity, userId)).build();
            //TODO CONTENT添加附件detail
        }
        //迄今时间
        themeQO.setUpToNowTime(calUpToNowTime(themeQO.getCreateTime()));
        //是否关注作者
        String authorId = themeQO.getAuthorId();
        Set<String> fansSet = new HashSet<>(followRelService.queryFansByFollowerId(userId));
        themeQO.setFollow(fansSet.contains(authorId));
        //用户信息：头像、昵称
        HomePageEntity userEntity = homePageService.selectByUserId(userId);
        themeQO.setUserImg(userEntity.getHeadImg());
        themeQO.setNickName(userEntity.getNickName());
        //热点数据:点赞，收藏，转发
        Integer likeCount = collectionService.getCountByTypeAndId(themeId, CollectionTypeEnum.LIKE_THEME);
        Integer bookCount = collectionService.getCountByTypeAndId(themeId, CollectionTypeEnum.BOOK_THEME);
        Integer forwardCount = themeService.getForwardCountById(themeId);
        themeQO.setCommentCount(bookCount);
        themeQO.setLikeCount(likeCount);
        themeQO.setForwardCount(forwardCount);
    }

    //组装正文详情
    private void buildMainTestExtraInfo(MainTextQo maintTextQo, String userId) {
        //附件列表
        List<ThemeAttachmentEntity> attachments = themeAttachmentService.selectByThemeId(maintTextQo.getThemeId());
        if (!CollectionUtils.isEmpty(attachments)) {
            List<AttachmentQo> attachmentQos = new ArrayList<>();
            attachments.forEach(o -> attachmentQos.add(AttachmentQo.builder()
                    .type(o.getAttachType())
                    .attachment(transferAttachment(o,userId))
                    .build()));
            maintTextQo.setAttachment(attachmentQos);
        }
        //迄今时间
        maintTextQo.setUpToNowTime(calUpToNowTime(maintTextQo.getCreateTime()));
        //用户信息：头像、昵称
        HomePageEntity author = homePageService.selectByUserId(userId);
        maintTextQo.setUserImg(author.getHeadImg());
        maintTextQo.setNickName(author.getNickName());

        return;
    }


    public Object transferAttachment(ThemeAttachmentEntity themeAttachment, String userId) {
        String attachmentId = themeAttachment.getAttachId();
        switch (themeAttachment.getAttachType()) {
            //附件类型 1:产品 2：直播 3：短视频 4：课程
            //TODO ENTITY 转 DTO
            case 1:
                CommonResp<ProductInfoVO> productDetailInfo = feignClientForProducts.getProductDetailInfo(attachmentId);
                return productDetailInfo.getData();
//                return finProResService.selectById(themeAttachment.getAttchId());

            case 2:
                CommonResp<ZhiboDetailVO> zhiboDetailVOCommonResp = feignClientForZhibo.queryZhiboDetail(attachmentId);
                return zhiboDetailVOCommonResp.getData();
//                return zhiboService.selectByid(themeAttachment.getAttchId());
            case 3:
                String[] attachmentIds = new String[]{attachmentId};
                CommonResp<List<ShortVideoBaseInfoResp>> shortVideoBaseInfo =
                        feignClientForTanpuroom.getShortVideoBaseInfo(Arrays.asList(attachmentIds));
                return shortVideoBaseInfo.getData().get(0);
//                CurriculumResEntity curriculumResEntity = curriculumResService.selectShortVideo(themeAttachment.getAttchId());
//                return curriculumResEntity;
            case 4:
                String[] split = attachmentId.split(",");
                if (split.length != 2) {
                    throw new BizException("课程id格式错误：" + attachmentId);
                }
                String courseId = split[0];
                String packageId = split[1];
                CommonResp<CourseDetailExt> courseIdDetailInfo = feignClientForCourse.getCourseIdDetailInfo(courseId, packageId);
                return courseIdDetailInfo.getData();

//                CurriCulumInfoDetailDto curriCulumInfoDetailDto = CurriCulumInfoDetailDto.builder().curriculumId(attachmentId).userId(userId).build();
//                CommonResp<CurriculumDetailVo> curriculumDetailVoCommonResp = feignClientForTanpuroom.queryCurriculumDetail(curriCulumInfoDetailDto);
//                return curriculumDetailVoCommonResp.getData();
//                curriculumResEntity = curriculumResService.selectById(themeAttachment.getAttchId());
//
//                OrderFlowEntity orderFlowEntity = orderFlowService.queryByUserAndProId(themeAttachment.getThemeId(), userId);
//                return curriculumResEntity;
            default:
                return null;


        }
    }


    //计算迄今时间
    private String calUpToNowTime(LocalDateTime start) {
        Duration between = Duration.between(start, LocalDateTime.now());
        long duration = between.toMinutes();
        if (duration < 1) {
            return "刚刚";
        } else if (duration < 60) {
            return duration + "分钟前";
        } else if (duration < 60 * 24) {
            return duration / 60 + "小时前";
        } else if (start.getYear() == LocalDateTime.now().getYear()) {
            return start.format(DateTimeFormatter.ofPattern("MM-dd HH:mm:ss"));
        }
        return start.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
    }


}
