package com.tanpu.community.service.other;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.tanpu.common.exception.BizException;
import com.tanpu.community.api.enums.BlockTypeEnum;
import com.tanpu.community.api.enums.DeleteTagEnum;
import com.tanpu.community.dao.entity.community.BlackListEntity;
import com.tanpu.community.dao.mapper.community.BlackListMapper;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

@Service
public class BlackListService {
    @Resource
    private BlackListMapper blackListMapper;

    public void addBlock(String blockId, String userId, BlockTypeEnum type) {

        BlackListEntity selected = selectOne(blockId, userId, type.getCode());
        //如果数据已存在，则修改delete_tag
        if (selected != null) {
            selected.setDeleteTag(DeleteTagEnum.NOT_DELETED.getCode());
            blackListMapper.updateById(selected);
        } else {
            BlackListEntity blackListEntity = BlackListEntity.builder()
                    .blocker(userId)
                    .blockedId(blockId)
                    .blockedType(type.getCode())
                    .build();
            blackListMapper.insert(blackListEntity);
        }

    }

    public BlackListEntity selectOne(String blockId, String blocker, Integer blockType) {
        return blackListMapper.selectOne(new LambdaQueryWrapper<BlackListEntity>()
                .eq(BlackListEntity::getBlocker, blocker)
                .eq(BlackListEntity::getBlockedId, blockId)
                .eq(BlackListEntity::getBlockedType, blockType)
                .eq(BlackListEntity::getDeleteTag, DeleteTagEnum.NOT_DELETED.getCode()));
    }

    public void removeBlackList(String blocker, String blockId, BlockTypeEnum blockType) {
        BlackListEntity selected = selectOne(blockId, blocker, blockType.getCode());
        if (selected == null) {
            throw new BizException("black_list数据不存在");
        } else {
            selected.setDeleteTag(DeleteTagEnum.DELETED.getCode());
            blackListMapper.updateById(selected);
        }
    }
}
